@@ -6,7 +6,10 @@ import declarative
 import textwrap
 import re
 import os
-from pkg_resources import resource_filename
+try:
+    from pkg_resources import resource_filename
+except ImportError:
+    resource_filename = None
 
 __all__ = ['NoDefault', 'Invalid', 'Validator', 'Identity',
            'FancyValidator', 'is_validator']
@@ -14,11 +17,36 @@ __all__ = ['NoDefault', 'Invalid', 'Validator', 'Identity',
 import gettext
 
 def get_localedir():
-    return resource_filename(__name__, "/i18n")
+    """
+    Retrieve the location of locales.
+
+    If we're built as an egg, we need to find the resource within the egg.
+    Otherwise, we need to look for the locales on the filesystem or in the
+    system message catalog.
+    """
+    locale_dir = ''
+    # Check the egg first
+    if resource_filename is not None:
+        try:
+            locale_dir = resource_filename(__name__, "/i18n")
+        except NotImplementedError:
+            # resource_filename doesn't work with non-egg zip files
+            pass
+    if os.access(locale_dir, os.R_OK | os.X_OK):
+        # If the resource is present in the egg, use it
+        return locale_dir
+
+    # Otherwise, search the filesystem
+    locale_dir = os.path.join(os.path.dirname(__file__), 'i18n')
+    if not os.access(locale_dir, os.R_OK | os.X_OK):
+        # Fallback on the system catalog
+        locale_dir = os.path.normpath('/usr/share/locale')
+
+    return locale_dir
 
 def set_stdtranslation(domain="FormEncode", languages=None, \
                        localedir = get_localedir()):
-    
+
     t = gettext.translation(domain=domain, \
                             languages=languages, \
                             localedir=localedir, fallback=True)
@@ -148,7 +176,8 @@ class Validator(declarative.Declarative):
     use_builtins_gettext = True #In case you dont want to use __builtins__._
                                 #altough it may be definied, set this to False
     
-    __singletonmethods__ = ('to_python', 'from_python')
+    __singletonmethods__ = ('to_python', 'from_python', 'message', 'all_messages',
+                            'subvalidators')
 
     def __classinit__(cls, new_attrs):
         if new_attrs.has_key('messages'):
@@ -352,6 +381,9 @@ class FancyValidator(Validator):
         try:
             if self.strip and isinstance(value, (str, unicode)):
                 value = value.strip()
+            elif hasattr(value, 'mixed'):
+                # Support Paste's MultiDict
+                value = value.mixed()
             if self.is_empty(value):
                 if self.not_empty:
                     raise Invalid(self.message('empty', state), value, state)
@@ -394,7 +426,7 @@ class FancyValidator(Validator):
                 if fp:
                     value = fp(value, state)
                 vo = self.validate_other
-                if vo and co is not self._validate_noop:
+                if vo and vo is not self._validate_noop:
                     vo(value, state)
                 return value
             else:
@@ -412,7 +444,8 @@ class FancyValidator(Validator):
 
     def is_empty(self, value):
         # None and '' are "empty"
-        return value is None or value == ''
+        return value is None or value == '' or (
+            isinstance(value, (list, tuple, dict)) and not value)
 
     def empty_value(self, value):
         return None
